/*
 *	cycarray.h
 *
 *	The Cyclic Array class is a modified version of the normal
 *	array, adding a modulo to the index.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_CYCARRAY_H_
#define	_OPaC_CYCARRAY_H_

#ifndef _OPaC_ARRAY_H_
#include <OPaC/array.h>
#endif

/****************************************************************/

/*
 *	Cyclic Array class.
 */

class OP_PUBLIC OPCyclicArray : public OPArray
{
	
protected:
	Count				modulo;					//	modulo for access
	
protected:
	OPCyclicArray ();
	virtual ~ OPCyclicArray ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
	virtual void InternalGetGapSize (Count, CountOut, CountOut); /* OVERRIDDEN */
	virtual void InternalSetGapSize (Count, Count, Count);		 /* OVERRIDDEN */

public:
	virtual Bool SetSize (Count size);						/* OVERRIDDEN */
	virtual OPObjectRef& Access (Count pos) const;			/* OVERRIDDEN */
	
	virtual Bool SetModulo (Count modulo);
	virtual Count ReturnModulo () const;
	
	OP_DEFINE_CLASS (OPCyclicArray, OPArray);
};

/****************************************************************/

#endif /* _OPaC_CYCARRAY_H_ */

