/*
 *	datarep.h
 *
 *	This file defines a class which reports data modifications to another
 *	object when it gets a change notification.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DATAREP_H_
#define	_OPaC_DATAREP_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_ATOM_H_
#include <OPaC/atom.h>
#endif

#ifndef _OPaC_STRING_H_
#include <OPaC/string.h>
#endif

/****************************************************************/

/*
 *	The Data Report class will report data modifications
 *	to another object which has not enough intelligence
 *	to do it itself.
 */

class OP_PUBLIC OPDataReport : public OPObject
{

protected:
	OPAtomRef			query_name;			//	name of data to query
	OPStringRef			query_path;			//	path of data to query
	OPStringRef			query_format;		//	format to apply to the queried data

	OPObjectRef			def_object;			//	object which will get the data
	OPAtomRef			def_name;			//	name of data to define
	OPStringRef			def_path;			//	path of data to define
	
	OPAtomRef			action_name;		//	action after reporting
	OPStringRef			action_path;		//	path to action after reporting
	
	Bool				is_visible;			//	TRUE => visible in data link browser
	
protected:
	OPDataReport ();
	virtual ~ OPDataReport ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void Initialise (OPObjectIn source, OPAtomIn event, OPAtomIn query,
							 OPObjectIn destination, OPAtomIn target,
							 OPAtomIn action = OPAtomNull,
							 Bool is_visible = TRUE);
	
	virtual void DefineQuery (OPAtomIn name, OPStringIn format = OPStringNull);
	virtual void DefineQueryPath (OPStringIn path, OPStringIn format = OPStringNull);
	
	virtual void DefineTarget (OPObjectIn obj, OPAtomIn name);
	virtual void DefineTargetPath (OPObjectIn obj, OPStringIn path);
	
	virtual void DefineAction (OPAtomIn name);
	virtual void DefineActionPath (OPStringIn path);
	
	const OPAtomRef& ReturnQueryName () const		{ return this->query_name; }
	const OPStringRef& ReturnQueryPath () const		{ return this->query_path; }
	const OPStringRef& ReturnQueryFormat () const	{ return this->query_format; }
	const OPObjectRef& ReturnTargetObject () const	{ return this->def_object; }
	const OPAtomRef& ReturnTargetName () const		{ return this->def_name; }
	const OPStringRef& ReturnTargetPath () const	{ return this->def_path; }
	const OPAtomRef& ReturnActionName () const		{ return this->action_name; }
	const OPStringRef& ReturnActionPath () const	{ return this->action_path; }
	
	virtual void SetVisible (Bool yes);
	virtual Bool IsVisible () const;
	
	virtual void Changed (OPObjectIn obj, Card32 id);
	
	OP_DEFINE_CLASS (OPDataReport, OPObject);
};

/****************************************************************/

#endif /* _OPaC_DATAREP_H_ */
