/*
 *	dataslot.h
 *
 *	This file defines the OPaC Data Slot class which is used to export data
 *	from an application to its user interface.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DATASLOT_H_
#define	_OPaC_DATASLOT_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

#ifndef _OPaC_ATOM_H_
#include <OPaC/atom.h>
#endif

/****************************************************************/

/*
 *	Define the Data Slot class. It maintains a list of the
 *	exported fields which can then be accessed by the user
 *	interface. An OPDataBase instance is used to store the
 *	data.
 */

class OP_PUBLIC OPDataSlot : public OPGenericContainer
{
	
protected:
	OPAtomRef			name;				//	slot name
	
	OPDataBaseRef		db_field;			//	data base containing the fields
	OPEntityModelRef	db_field_model;		//	data model for the fields

	OPDataBaseRef		db_control;			//	data base containing the controllers
	OPEntityModelRef	db_control_model;	//	data model for the controllers

	Card32				how_to_extract;		//	extraction mode to be used with "Get"

protected:
	OPDataSlot ();
	virtual ~ OPDataSlot ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Count TotalCount () const;					/* OVERRIDDEN */
	virtual Bool Get (Count index, OPObjectOut object);	/* OVERRIDDEN */
	virtual Bool SpecifyExtraction (OPArrayIn x);		/* OVERRIDDEN */
	
	virtual void SetName (OPAtomIn name);
	virtual Bool GetName (OPAtomOut name);
	
	virtual void AddField (OPAtomIn name, OPObjectIn data);
	virtual void AddField (OPAtomIn name, OPObjectIn data, OPTargetIn observer);
	virtual void AddController (OPAtomIn name, int controller);
	
	virtual Bool FindField (OPAtomIn name, OPObjectOut data);
	virtual Bool FindField (OPAtomIn name, OPObjectOut data, OPTargetOut observer);
	virtual Bool FindController (OPAtomIn name, int controller);

	virtual OPDataBaseRet ReturnFieldBase () const;
	virtual OPDataBaseRet ReturnControllerBase () const;

	OP_DEFINE_CLASS (OPDataSlot, OPGenericContainer);
};

#endif /* _OPaC_DATASLOT_H_ */
