/*
 *	dataview.h
 *
 *	This file defines a view class which displays the data represented by
 *	the associated object. The data has to be represented using a string.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DATAVIEW_H_
#define	_OPaC_DATAVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_MONITORVALUE_H_
#include <OPaC/monitorvalue.h>
#endif

/****************************************************************/

/*
 *	The Data View class will display some data in a view
 *	using the specified type, format and font.
 */

class OP_PUBLIC OPDataView : public OPMonitorValue
{

protected:
	OPTextTagBaseRef	text_tag;			//	associated text tag
	OPStringRef			text;				//	text (data)
	OPStringRef			format;				//	format specifier for the data
	
protected:
	OPDataView ();
	virtual ~ OPDataView ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void BuildTextFromData ();
	
	virtual void DoUpdate ();							/* OVERRIDDEN */
	virtual void ChangeGeometry (OPRectIn box);			/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool InitTextTag (OPTextTagBaseIn special_text_tag);
	virtual Bool InitValueFormat (OPStringIn format);
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const; /* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPDataView, OPMonitorValue);
};

/****************************************************************/

#endif /* _OPaC_DATAVIEW_H_ */
