/*
 *	dictree.h
 *
 *	The Dictionary Tree is a simple implementation of a dictionary
 *	like tree structure which can be accessed using paths instead of
 *	simple keys.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DICTREE_H_
#define	_OPaC_DICTREE_H_

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

#ifndef _OPaC_ITERATOR_H_
#include <OPaC/iterator.h>
#endif

/****************************************************************/

/*
 *	The Dictionary Tree class has the same interface as a flat
 *	dictionary, but it handles its keys like paths (arrays of
 *	keys).
 */

class OP_PUBLIC OPDicTree : public OPSet
{
	friend class OPDicTreeIterator;
	
protected:
	OPDictRef			root;					//	root of the tree
	UnicodeChar			separator;				//	default separator
	
protected:
	OPDicTree ();
	virtual ~ OPDicTree ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual Bool FindRootDict (OPObjectIn path, OPObjectOut key, OPDictOut dict);
	virtual Bool FindSubTree (OPDictIn dict, Count pos, OPDictOut found, CountOut index, OPListIn path);
	virtual void RemoveSubTree (OPDictIn dict);
	
	virtual Bool SplitTextPath (const LatinChar* path, OPListOut list);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	virtual OPIteratorRef NewIterator ();				/* OVERRIDDEN */
	
	virtual Bool Insert (OPObjectIn k, OPObjectIn d);	/* OVERRIDDEN */
	virtual Bool Remove (OPObjectIn k, Count = 0);		/* OVERRIDDEN */
	virtual Bool Remove (Count index);					/* OVERRIDDEN */
	virtual void RemoveAll ();							/* OVERRIDDEN */
	virtual Bool RemoveValue (OPObjectIn v, Count = 0);	/* OVERRIDDEN */
	
	virtual Bool Get (Count, OPObjectOut, OPObjectOut);	/* OVERRIDDEN */
	virtual Bool GetFromKey (OPObjectIn key_model, OPObjectOut data,
							 Count nth = 0);			/* OVERRIDDEN */
	virtual Bool GetFromValue (OPObjectIn value_model, OPObjectOut key,
							   Count nth = 0);			/* OVERRIDDEN */
	
	virtual Bool LookFor (OPObjectIn value);			/* OVERRIDDEN */
	
	virtual Bool SetPathSeparator (UnicodeChar sep);
	virtual UnicodeChar ReturnPathSeparator () const;
	
	virtual Bool InsertUsingPath (OPStringIn path, OPObjectIn data);
	virtual Bool InsertUsingPath (const LatinChar* path, OPObjectIn data);
	virtual Bool GetFromPath (OPStringIn path, OPObjectOut data);
	virtual Bool GetFromPath (const LatinChar* path, OPObjectOut data);
	
	OP_DEFINE_CLASS (OPDicTree, OPSet);
};

/****************************************************************/

/*
 *	Dictionary tree iterator.
 */

class OP_PUBLIC OPDicTreeIterator : public OPIterator
{
	
protected:
	
	
protected:
	OPDicTreeIterator ();
	virtual ~ OPDicTreeIterator ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	//	The iterator exports following overridden methods :
	
	virtual Bool First ();								/* OVERRIDDEN */
	virtual Bool Last ();								/* OVERRIDDEN */
	
	virtual Bool Next (Count move = 1);					/* OVERRIDDEN */
	virtual Bool Prev (Count move = 1);					/* OVERRIDDEN */
	
	virtual Bool Get (OPObjectOut, Count index = 0);	/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPDicTreeIterator, OPIterator);
};

/****************************************************************/

#endif /* _OPaC_DICTREE_H_ */
