/*
 *	dragview.h
 *
 *	The drag view is provided in order to be able to move views around the
 *	screen and drop them on other OPaC views.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_DRAGVIEW_H_
#define	_OPaC_DRAGVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_WINDOW_H_
#include <OPaC/window.h>
#endif

/****************************************************************/

/*
 *	The drag view is a special kind of window. Just put the
 *	view you want to drag around into the window...
 */

class OP_PUBLIC OPDragView : public OPWindow
{
	
protected:
	OPViewRef			drag_view;				//	view which is being dragged
	OPObjectRef			drag_object;			//	object which is being dragged (if any)
	OPViewState			old_state;				//	state of view before dragging
	OPViewRef			drop_view;				//	view where dropping would happen
	OPRect				drop_box;				//	box where dropping would happen
	Bool				dragging;				//	TRUE => currently dragging
	Bool				auto_stop;				//	TRUE => stop dragging automatically
	Bool				auto_oops;				//	TRUE => bring back if released anywhere
	Bool				auto_embed;				//	TRUE => embed view when dropping
	Bool				notifying_drop;			//	TRUE => notifying drop to observers
	Bool				is_hot_spot_ok;			//	TRUE => [ox;oy] defined
	OPCoord				start_x;				//	[x] before start
	OPCoord				start_y;				//	[y] before start
	OPCoord				ox;						//	[x] offset
	OPCoord				oy;						//	[y] offset
	OPCoord				drop_x;					//	[x] drop position
	OPCoord				drop_y;					//	[y] drop position
	
	Bool				drop_only_in_view;		//	TRUE => drop only in specified view
	OPRect				drop_only_rect;			//	rectangle where dropping is enabled
	OPWindowRef			drop_only_window;		//	window for the corresponding rectangle
	Bool				drop_cancelled;			//	TRUE => drop cancelled when committing
	
protected:
	OPDragView ();
	virtual ~ OPDragView ();

	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */

	virtual Bool FindDropView ();
	virtual void DragEnd ();
	
public:
	virtual void Setup (OPRectIn box);
	virtual void Setup (OPViewIn view, Bool auto_embed = FALSE);
	
	virtual void DropOnlyInView (OPViewIn view);
	
	virtual Bool GetDropOrigin (OPCoordOut x, OPCoordOut y) const;
	virtual Bool GetDropView (OPViewOut view) const;
	virtual Bool GetDropBox (OPRectOut box) const;
	
	virtual void SetAutoStop (Bool yes);
	virtual void SetAutoOops (Bool yes);
	virtual void SetAutoEmbed (Bool yes);
	virtual void SetHotSpot (OPCoord x, OPCoord y);
	
	virtual void DragBegin (OPViewIn view);
	virtual void DragForget ();
	virtual void DragDrop ();
	virtual void DragOops ();
	
	virtual Bool SetDragObject (OPObjectIn object);
	virtual OPObjectRet ReturnDragObject () const;
	
	virtual void CommitDrop (OPObjectIn target);
	virtual void CommitConnect (OPObjectIn target);
	virtual void SubstituteTarget (OPObjectIn target);
	
	virtual void Delete ();								/* OVERRIDDEN */
	virtual void ButtonRelease (OPEventIn event);		/* OVERRIDDEN */
	virtual void DragNotify (OPEventIn event);			/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (OPObject, drag_object, SetDragObject, ReturnDragObject);
	OP_DEFINE_CLASS (OPDragView, OPWindow);
};

/****************************************************************/

#endif /* _OPaC_DRAGVIEW_H_ */
