/*
 *	memory.cxx
 *	
 *	Interface to system dependent memory handling functions. This should only
 *	be used by the OPMemZone class, not by the application developer.
 *	
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_MEMORY_H_
#define	_OPaC_MEMORY_H_

#ifndef _OPaC_TYPES_H_
#include <OPaC/types.h>
#endif


/*
 *	OPaC relies on the fact that the underlying system may
 *	allocate memory in different zones. These zones may be
 *	used to group memory allocations in order to speed up
 *	accesses to contiguous virtual memory locations.
 *
 *	In case the underlying system does not support this
 *	concept, it will have to mimick the zones. That should
 *	not be too hard to implement :-)
 */

extern Bool SysMemOpenZone (const LatinChar* name,
							Card32Out zone_id);


/*
 *	Closing a memory zone might free the memory which has
 *	not been explicitely freed by the programmer. But this
 *	is system dependent, so be careful to free everything
 *	yourself !
 */

extern Bool SysMemCloseZone (Card32In zone_id);


/*
 *	The memory allocation function returns TRUE on success
 *	and sets allocated >= size to the real number of bytes
 *	which has been returned by the system.
 *
 *	The zone id specifies from which memory pool the system
 *	should try to allocate RAM.
 *
 *	The function call might return some special memory ID
 *	which will be needed for deallocation (if not 0).
 */

extern Bool SysMemAlloc (Card32In zone_id, Card32In size,
						 BytePtrOut ptr,
						 Card32Out allocated,
						 Card32Out mem_id);


/*
 *	The memory free function will have to free the memory
 *	
 */

extern Bool SysMemFree (Card32In zone_id,
						BytePtrIn ptr,
						Card32In allocated,
						Card32In mem_id);

#endif	_OPaC_MEMORY_H_

