/*
 *	plotview.h
 *
 *	Plot View class definition.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_PLOTVIEW_H_
#define	_OPaC_PLOTVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

enum OPPlotViewType
{
	OP_PLOT_VIEW_NONE = 0,					//	no data
	OP_PLOT_VIEW_Y_T  = 1,					//	data consists of [y] samples
	OP_PLOT_VIEW_X_Y  = 2					//	data consists of [x,y] pairs
};

OP_TYPEENUM (OPPlotViewType);

/****************************************************************/

/*
 *	The PlotView class represents data sets graphically.
 */

class OP_PUBLIC OPPlotView : public OPView
{
	
protected:
	Real				h_zero;				//	horizontal offset for zero [0..1]
	Real				h_size;				//	horizontal (real) size
	
	Real				v_zero;				//	vertical offset for zero [0..1]
	Real				v_size;				//	vertical (real) size
	
	OPColorPixel		default_color;		//	default color for drawing data set
	
	OPPlotViewType		data_type;			//	data type for representation
	Real*				data_set;			//	data set (array of numbers or pairs)
	Count				data_num;			//	number of reals in data set
	Count				data_org;			//	offset of origin in data set (for wrap around)
	OPMemZone*			data_zone;			//	zone to free from (if any)
	
protected:
	OPPlotView ();
	virtual ~ OPPlotView ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
	virtual void StoreDataSet (OPStorageIn storage);
	virtual void RestoreDataSet (OPStorageIn storage);
	virtual void FlushDataSet ();
	
public:
	virtual void Delete ();									/* OVERRIDDEN */
	
	virtual void DefineDataSet (const Real* set, Count num, OPPlotViewType type, OPMemZone* zone = 0);
	virtual void CloneDataSet ();
	
	virtual Count TotalCount () const;
	virtual Bool GetDataPoint (Count index, RealOut x, RealOut y, OPColorPixelOut color);
	
	virtual Bool SetData (Count pos, Real);
	virtual Bool SetData (Count pos, Real, Real);
	virtual Bool GetData (Count pos, RealOut) const;
	virtual Bool GetData (Count pos, RealOut, RealOut) const;
	
	virtual Bool SetHorizontalZero (Real zero);
	virtual Bool SetHorizontalSize (Real size);
	virtual Bool SetVerticalZero (Real zero);
	virtual Bool SetVerticalSize (Real size);
	virtual Bool SetDataOrigin (Count origin);
	
	virtual Real ReturnHorizontalZero () const;
	virtual Real ReturnHorizontalSize () const;
	virtual Real ReturnVerticalZero () const;
	virtual Real ReturnVerticalSize () const;
	virtual Count ReturnDataOrigin () const;
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (Real, h_zero, SetHorizontalZero, ReturnHorizontalZero);
	OP_DEFINE_SLOT (Real, h_size, SetHorizontalSize, ReturnHorizontalSize);
	OP_DEFINE_SLOT (Real, v_zero, SetVerticalZero, ReturnVerticalZero);
	OP_DEFINE_SLOT (Real, v_size, SetVerticalSize, ReturnVerticalSize);
	
	OP_DEFINE_CLASS (OPPlotView, OPView);
};

/****************************************************************/

#endif /* _OPaC_PLOTVIEW_H_ */
