/*
 *	pubmacro.h
 *
 *	This exports the macros which will be used to publish information
 *	about queriable data.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_PUBMACRO_H_
#define	_OPaC_PUBMACRO_H_

#ifndef _OPaC_VALUES_H_
#include <OPaC/values.h>
#endif

/****************************************************************/

/*
 *	The accessors and modifiers are defined as pointers to
 *	member functions, one for each supported type :
 */

typedef OPValueRef (*OPaC_ReadAccessValue)(const OPObject*);
typedef Bool (*OPaC_WriteAccessValue)(OPObject*, OPValueIn);

/****************************************************************/

/*
 *	Data can be published thanks to the following macros :
 *
 *	- OP_PUBLISH_BITSET_INFO (Class, "name", info);
 *
 *	- OP_PUBLISH_SLOT (Class, "name", Get, Set, "desc.", Type);
 *	- OP_PUBLISH_SLOT_READ (Class, "name", Get, "desc.", Type);
 *	- OP_PUBLISH_SLOT_WRITE (Class, "name", Set, "desc.", Type);
 *
 *	The parameters are defined as following :
 *
 *	- Class is the class name for which the publishing is done.
 *	- "name" is the name of the data field.
 *	- Get/Set are the methods for getting or retrieving the field.
 *	- "desc." is a short description of the data field.
 *	- Type is the type of the data field.
 */

#define	OP_PUBLISH_BITSET_INFO(c,name,b)						\
	OP_PUBLISH_BITSET_INFO_1(c,name,b,__LINE__)

#define	OP_PUBLISH_SLOT(c,name,get,set,desc,type)				\
	OP_PUBLISH_GENERAL_SLOT_1(c,name,get,set,desc,type,__LINE__)

#define	OP_PUBLISH_SLOT_READ(c,name,get,desc,type)				\
	OP_PUBLISH_READ_SLOT_1(c,name,get,desc,type,__LINE__)

#define	OP_PUBLISH_SLOT_WRITE(c,name,set,desc,type)				\
	OP_PUBLISH_WRITE_SLOT_1(c,name,set,desc,type,__LINE__)

/****************************************************************/

#define	OP_PUBLISH_BITSET_INFO_1(c,name,b,l)					\
	OP_PUBLISH_BITSET_INFO_2(c,name,b,l)

#define	OP_PUBLISH_GENERAL_SLOT_1(c,name,get,set,desc,type,l)	\
	OP_PUBLISH_GENERAL_SLOT_2(c,name,get,set,desc,type,l)

#define	OP_PUBLISH_READ_SLOT_1(c,name,get,desc,type,l)			\
	OP_PUBLISH_READ_SLOT_2(c,name,get,desc,type,l)

#define	OP_PUBLISH_WRITE_SLOT_1(c,name,set,desc,type,l)			\
	OP_PUBLISH_WRITE_SLOT_2(c,name,set,desc,type,l)

/****************************************************************/

#define	OP_PUBLISH_BITSET_INFO_2(c,name,b,l)					\
	OP_PUBLISH__B0(c,l,name);									\
	notify->PublishDataEx (pdd, b, sizeof (b));					\
	OP_PUBLISH__EX(c,l)

/****************************************************************/

#define	OP_PUBLISH_GENERAL_SLOT_2(c,name,get,set,desc,type,l)	\
	OP_PUBLISH__RDACC(c,l,type,get);							\
	OP_PUBLISH__WRACC(c,l,type,set);							\
	OP_PUBLISH__B(c,l,name,desc,type);							\
	pdd.is_read_ok   = TRUE;									\
	pdd.is_write_ok  = TRUE;									\
	pdd.access_read  = __rd_acc_##c##_##l;						\
	pdd.access_write = __wr_acc_##c##_##l;						\
	OP_PUBLISH__E(c,l)

#define	OP_PUBLISH_READ_SLOT_2(c,name,get,desc,type,l)			\
	OP_PUBLISH__RDACC(c,l,type,get);							\
	OP_PUBLISH__B(c,l,name,desc,type);							\
	pdd.is_read_ok   = TRUE;									\
	pdd.is_write_ok  = FALSE;									\
	pdd.access_read  = __rd_acc_##c##_##l;						\
	pdd.access_write = 0;										\
	OP_PUBLISH__E(c,l)

#define	OP_PUBLISH_WRITE_SLOT_2(c,name,set,desc,type,l)			\
	OP_PUBLISH__WRACC(c,l,type,set);							\
	OP_PUBLISH__B(c,l,name,desc,type);							\
	pdd.is_read_ok   = FALSE;									\
	pdd.is_write_ok  = TRUE;									\
	pdd.access_read  = 0;										\
	pdd.access_write = __wr_acc_##c##_##l;						\
	OP_PUBLISH__E(c,l)

/****************************************************************/

#define OP_PUBLISH__WRACC(c,l,type,set)							\
static Bool														\
__wr_acc_##c##_##l (OPObject* obj, OPValueIn data)				\
{																\
	c* instance = (c*)(obj);									\
	return instance->set (OPaC_ValueTo##type (data));			\
}																\
class OPObject

#define OP_PUBLISH__RDACC(c,l,type,get)							\
static OPValueRef												\
__rd_acc_##c##_##l (const OPObject* obj)						\
{																\
	const c* instance = (const c*)(obj);						\
	return instance->get ();									\
}																\
class OPObject

#define	OP_PUBLISH__B0(c,l,name)								\
static void														\
__collect_##c##_##l (OPRuntime* notify)							\
{																\
	OPaC_PubDataDef pdd = { 0 };								\
	pdd.runtime     = & OP_RUNTIME (c);							\
	pdd.var_name    = #name

#define	OP_PUBLISH__B(c,l,name,desc,type)						\
	OP_PUBLISH__B0(c,l,name);									\
	pdd.var_desc    = desc;										\
	pdd.var_type    = type##TypeDef

#define	OP_PUBLISH__E(c,l)										\
	notify->PublishData (pdd);									\
}																\
static OPaC_PubDataExport										\
__export_##c##_##l (__collect_##c##_##l)

#define	OP_PUBLISH__EX(c,l)										\
}																\
static OPaC_PubDataExport										\
__export_##c##_##l (__collect_##c##_##l)

/****************************************************************/

enum OPBitsetDataType
{
	OP_BITSET_DATA_MASK		= 0,
	OP_BITSET_DATA_BIT		= 1,
	OP_BITSET_DATA_VALUE	= 2
};

OP_TYPEENUM (OPBitsetDataType);

/****************************************************************/

struct OPaC_BitsetDataDef
{
	Card32				value;					//	value (either mask/bit/value)
	const LatinChar*	name;					//	name of this element
	OPBitsetDataType	type;					//	type (either mask/bit/value)
};

/****************************************************************/

/*
 *	The public data declaration needs some special structures
 *	which won't be used directly by the programmer, but will
 *	be generated by the OP_PUBLISH_xxx macros.
 */

struct OPaC_PubDataDef
{
	const OPRuntimeRef*	runtime;				//	runtime description
	const LatinChar*	var_name;				//	variable name
	const LatinChar*	var_desc;				//	variable description
	OPTypeDef			var_type;				//	variable type
	Bool				is_read_ok;				//	TRUE => may read from variable
	Bool				is_write_ok;			//	TRUE => may write to variable
	OPaC_ReadAccessValue  access_read;			//	pointer to read accessor
	OPaC_WriteAccessValue access_write;			//	pointer to write accessor
};

typedef void (*OPaC_PubDataInFunc)(OPRuntime*);

/*
 *	The published data export class will record the individual
 *	data definitions which are then handled by OPRuntime.
 */

class OP_PUBLIC OPaC_PubDataExport
{
	friend class OPRuntime;
	
private:
	static OPaC_PubDataExport* head;			//	first published data
	static OPaC_PubDataExport* tail;			//	last published data
	
private:
	OPaC_PubDataExport*	next;					//	next published data in global list
	OPaC_PubDataInFunc	init;					//	published data initialization function
	
public:
	OPaC_PubDataExport (OPaC_PubDataInFunc init)
		{ (this->tail ? this->tail->next : this->head) = this;
		  this->init = init; this->next = 0; this->tail = this; }
};

/****************************************************************/

#endif /* _OPaC_PUBMACRO_H_ */
