/*
 *	random.h
 *
 *	Generate a uniformly distributed (pseudo-)random number.
 *	The generator is not platform dependent.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_RANDOM_H_
#define	_OPaC_RANDOM_H_

#ifndef _OPaC_VALUES_H_
#include <OPaC/values.h>
#endif

/****************************************************************/

OP_EXPORT (OPRandom, OPValue);					//	<OPaC/random.h>
OP_DEFINE_ARRAY (OPRandom, OPValue);			//	<OPaC/random.h>

/****************************************************************/

/*
 *	The random class has the same interface as the value
 *	wrapper and can thus be used very easily. Every access
 *	produces a new random value.
 */

class OP_PUBLIC OPRandom : public OPValue
{
	
private:
	mutable Card32		accumulate[55];
	mutable Card32		index;
	
protected:
	Card32				max;					//	maximum value generated [0..n]
	Card32				max_round;				//	largest multiple of "max_out"
	Card32				max_modulo;				//	n+1 for modulo arithmetic
	
	Real				scale;
	Real				offset;
	
protected:
	OPRandom ();
	virtual ~ OPRandom ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void RefreshValue () const;					/* OVERRIDDEN */
	virtual Card32 ReturnInitialSeed ();
	
	static Card32 Multiply (Card32, Card32);
	
public:
	virtual void Initialise ();
	
	virtual Bool SetMaximum (Card32 max);
	virtual Bool SetScale (Real scale);
	virtual Bool SetOffset (Real offset);
	
	virtual Bool SetScaleOffset (Real scale, Real offset);
	
	virtual Card32 ReturnMaximum () const;
	virtual Real   ReturnScale () const;
	virtual Real   ReturnOffset () const;
	
	OP_DEFINE_SLOT (Card32, maximum, SetMaximum, ReturnMaximum);
	OP_DEFINE_SLOT (Real, scale, SetScale, ReturnScale);
	OP_DEFINE_SLOT (Real, offset, SetOffset, ReturnOffset);
	OP_DEFINE_CLASS (OPRandom, OPValue);
};

/****************************************************************/

#endif /* _OPaC_RANDOM_H_ */

