/*
 *	scrlistpane.h
 *
 *	The Scroll List Pane provides a place holder for a horizontal
 *	list of panes.
 *
 *	(C) Copyright 1997, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SCRLISTPANE_H_
#define	_OPaC_SCRLISTPANE_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

/****************************************************************/

/*
 *	The Scroll List Pane is a place holder for a horizontal
 *	list of panes.
 */

class OP_PUBLIC OPScrollListPane : public OPControl
{
	
protected:
	OPScrollerRef		scroller;			//	horizontal scroller
	OPCoord				spacing;			//	spacing between scroll lists
	OPCoord				caption_height;		//	height of caption
	
	OPViewArray			view_array;			//	array of views (panes)
	OPTextCaptionArray	caption_array;		//	array of captions
	
	Count				total_num;			//	# of scroll lists
	Count				visible_num;		//	# of visible scroll lists
	Count				left_index;			//	index of leftmost visible scroll list
	
	Bool				is_geom_ok;			//	TRUE => geometry is OK
	
protected:
	OPScrollListPane ();
	virtual ~ OPScrollListPane ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn);				/* OVERRIDDEN */
	virtual void UpdateGeometry ();
	
	virtual void UpdateScroller ();
	virtual void UpdateColumns ();
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool SetVisibleColumns (Count num);
	virtual Bool SetTotalColumns (Count max);
	
	virtual Bool SetScroller (OPScrollerIn scroller);
	virtual Bool SetContents (Count index, OPViewIn view);
	virtual Bool SetCaption (Count index, OPTextCaptionIn caption);
	
	virtual Bool ShowColumn (Count index, int relative = 0);
	
	//	Observers exported for run-time binding :
	
	virtual void HandleScroll (OPObjectIn object, Card32 id);
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	virtual Bool Detect (OPCoord x, OPCoord y, OPViewOut);				/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPScrollListPane, OPControl);
};

#endif /* _OPaC_SCRLISTPANE_H_ */
