/*
 *	selgrid.h
 *
 *	The Selection Grid class allows a user to select one or several cells
 *	in a Cell Grid.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SELGRID_H_
#define	_OPaC_SELGRID_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

/****************************************************************/

#ifndef _OPaC_CELLGRID_H_
#include <OPaC/cellgrid.h>
#endif

/****************************************************************/

/*
 *	The SelGrid is an extended Cell Grid which allows the
 *	user to select one or several cells in the grid.
 */

class OP_PUBLIC OPSelGrid : public OPCellGrid
{
	
protected:
	Count				select_num;				//	# of selected cells
	
	Bool				select_on_click;		//	TRUE => clicking selects elements
	Bool				select_multiple_ok;		//	TRUE => allows multiple selections
	Bool				select_at_least_one;	//	TRUE => at least one selection
	Bool				search_by_lines;		//	TRUE => search line by line
	
	Bool				insert_between_lines;	//	TRUE => insert between lines
	Bool				insert_between_columns;	//	TRUE => insert between columns
	Bool				insert_over_cell;		//	TRUE => insertion over a cell is possible
	
	Bool				enable_cell_drag;		//	TRUE => cell dragging detection enabled
	Bool				is_dragging;			//	TRUE => dragging in progress
	Bool				is_drop_valid;			//	TRUE => valid drop position (h/v)
	Bool				is_drop_done;			//	TRUE => drop done in this grid
	Bool				is_drag_done;			//	TRUE => drag done in this grid
	
	Count				drag_source_h;			//	horizontal position of drag source
	Count				drag_source_v;			//	vertical position of drag source
	Count				drop_h;					//	horizontal position of drop destination
	Count				drop_v;					//	vertical position of drop destination
	Real				h_percent;				//	horizontal % in cell
	Real				v_percent;				//	vertical % in cell
	int					draw_overlay;			//	overlay which will be drawn
	
	OPDragViewRef		drag_view;				//	drag view used when dragging
	
protected:
	OPSelGrid ();
	virtual ~ OPSelGrid ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void NotifyArraySize (Count dx, Count dy);	/* OVERRIDDEN */
	virtual void NotifyAttachCell (OPControlIn cell, Count x, Count y);/* OVERRIDDEN */
	virtual void NotifyDetachCell (OPControlIn cell, Count x, Count y);/* OVERRIDDEN */
	
	virtual Bool SetInternalActiveHilite (Count x, Count y, Bool yes, Bool report);
	virtual void ChangeCellObserver (OPAtomIn event, OPAtomIn actor, Bool set);
	
public:
	virtual Bool SetActiveHilite (Count x, Count y, Bool yes, Bool report = TRUE);
	virtual Bool RemoveEveryHilite ();
	virtual Bool ReturnActiveHilite (Count x, Count y) const;
	
	virtual Bool EnableDragging (Bool yes);
	virtual Bool ReturnDraggingEnabled () const;
	virtual Bool IsDragging () const;
	
	virtual Bool SetSelectOnClick (Bool yes);
	virtual Bool SetSelectMultipleOk (Bool yes);
	virtual Bool SetSelectAtLeastOne (Bool yes);
	virtual Bool SetSearchLinesFirst (Bool yes);
	virtual Bool SetInsertBetweenLines (Bool yes);
	virtual Bool SetInsertBetweenColumns (Bool yes);
	virtual Bool SetInsertOverCell (Bool yes);
	
	virtual Bool ReturnSelectOnClick () const;
	virtual Bool ReturnSelectMultipleOk () const;
	virtual Bool ReturnSelectAtLeastOne () const;
	virtual Bool ReturnSearchLinesFirst () const;
	virtual Bool ReturnInsertBetweenLines () const;
	virtual Bool ReturnInsertBetweenColumns () const;
	virtual Bool ReturnInsertOverCell () const;
	
	virtual Bool GetDropPosition (CountOut h, CountOut v, RealOut ph, RealOut pv) const;
	virtual Bool GetDragOrigin (CountOut h, CountOut v) const;
	virtual void ResetDragDropStatus ();
	
	virtual Count ReturnSelectNum () const;
	
	virtual Bool FindActiveHilite (CountOut x, CountOut y, Count nth = 0) const;
	
	//	Observers exported for run-time binding :
	
	virtual void HandleCellClick (OPObjectIn src, Card32 id);
	virtual void HandleDragBegin (OPObjectIn src, Card32 id);
	virtual void HandleDragCommit ();
	virtual void HandleDragCancel ();
	
	virtual Bool DragAndDrop (OPRectIn, OPObjectIn, OPDragDropProtocol);	/* OVERRIDDEN */
	virtual void DrawOverlay (OPGrailPortIn port, OPRectIn rect);			/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPSelGrid, OPCellGrid);
};

#endif /* _OPaC_SELGRID_H_ */

