/*
 *	valueset.h
 *
 *	The classes used to represent simple values derive from OPValue
 *	class. This allows a simple grouping of these values into sets
 *	and other (possibly ordered) collection classes.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_VALUE_H_
#define	_OPaC_VALUE_H_

#include <OPaC/object.h>


class OPSet;
class OPSetBrowser:


/*
 *	A Set is an abstract collection of Value items. This offers
 *	methods to insert and remove value objects. Access to some
 *	robust browser is also available.
 */

class OPSet : public OPObject
{
	friend class OPSetBrowser;
	
	OP_DECLARE_CLASS (OPSet);
	
private:
	OPSetBrowser*		browse_first;			//	first active browser
	OPSetBrowser*		browse_last;			//	last active browser
	
protected:
	virtual void SaveInstance (OPArchive&) const;
	virtual void RestoreInstance (OPArchive&);
	
public:
	virtual void Insert (const OPValue*);
	virtual Bool Remove (const OPValue*);
	virtual Bool Find (const OPValue*) const;
	
	virtual OPSetBrowser* NewBrowser ();
};



/*
 *  A Set Browser is an abstract iterator used to walk through
 *	a collection of values stored in an OPaC set class.
 *
 *	The browser is considered to be robust, since it will try
 *	to catch up with insertions and deletions of items in the
 *	collection.
 */

class OPSetBrowser : public OPObject
{
	friend class OPSet;
	
	OP_DECLARE_CLASS (OPSetBrowser);
	
private:
	OPSetBrowser*		browse_next;			//	pointer to next browser (same set)
	OPSetBrowser*		browse_prev;			//	pointer to previous browse (same set)
	OPSet*				set;					//	pointer to containing set
	
protected:
	Count				position;				//	position counter (for emergencies)
	
protected:
	OPSetBrowser (OPSet*);
	
	virtual void SaveInstance (OPArchive&) const;
	virtual void RestoreInstance (OPArchive&);
	
public:
	virtual ~ OPSetBrowser ();
	
	virtual Bool Next () = 0;
	virtual Bool Prev () = 0;
	
	virtual const OPValue* Access () const = 0;
};


#endif /* _OPaC_VALUE_H_ */
