/*
 *	winstyle.h
 *
 *	Defines the window styles.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_WINSTYLE_H_
#define	_OPaC_WINSTYLE_H_

/*
 *	Enumerate the possible window styles. These specify the
 *	adornments a window manager is allowed to add.
 */

OP_TYPEDEF (Card32, OPWindowStyle);

//	The following are modes and only one of these can be applied
//	simultaneously :
	
#define	OP_WIN_NAKED		0x0000				//	naked window (no frame)
#define	OP_WIN_NORMAL		0x0001				//	normal window without title
#define	OP_WIN_TITLE		0x0002				//	normal window with title
	
//	The following are attributes and can be combined together with
//	the mode to describe the window's behaviour :
	
#define	OP_WIN_MOVE_OK		0x0010				//	window may be moved
#define	OP_WIN_SIZE_OK	 	0x0020				//	window may be sized
#define	OP_WIN_REDUCE_OK	0x0040				//	window may be reduced (iconified)
#define	OP_WIN_CLOSE_OK		0x0080				//	window may be closed

//	Extended attributes which should be understood by every port of
//	OPaC but obviously can't.

#define	OP_WIN_TRANSPARENT	0x00010000			//	window should be transparent
#define	OP_WIN_FLOAT_TOOL	0x00020000			//	window should be considered as a tool
#define	OP_WIN_TOP_LEVEL	0x00040000			//	window should be always on top
#define	OP_WIN_PARTITION	0x00080000			//	window should be partitioned
	
//	Masks :
	
#define	OP_WIN_MODE_MASK	0x0000000F			//	mask for mode
#define	OP_WIN_ATTR_MASK	0x0000FFF0			//	mask for attributes
#define	OP_WIN_EXTEND_MASK	0xFFFF0000			//	mask for extended attributes

#endif /* _OPaC_WINSTYLE_H_ */
