/*
 *	dictit.cxx
 *
 *	The Dictionary needs a dedicated Iterator class.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/container.h>
#include <OPaC/dictit.h>
#include <OPaC/new.h>

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPDictIterator);

OPDictIterator::OPDictIterator ()
{
	this->current  = 0;
}

OPDictIterator::~OPDictIterator ()
{
}

void
OPDictIterator::StoreInstance (OPStorageIn storage)
{
	super::StoreInstance (storage);
	static OPStorageSync info ("<OPDictIterator>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << OPStorageSync::end;
}

void
OPDictIterator::RestoreInstance (OPStorageIn storage)
{
	super::RestoreInstance (storage);
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> OPStorageSync::end;
}


void
OPDictIterator::NotifyInsert (OPDict::Element* elem)
{
}


void
OPDictIterator::NotifyRemove (OPDict::Element* elem)
{
	if (this->current == elem) {
		if (elem->next) {
			this->current = elem->next;
		} else if (elem->prev) {
			this->current = elem->prev;
		} else {
			this->current = 0;
		}
		this->did_change = TRUE;
	}
}


void
OPDictIterator::Delete ()
{
	super::Delete ();
}

Bool
OPDictIterator::First ()
{
	OPDictRef        dict  = this->source;
	OPDict::Element* first = dict->head;
	
	if (this->current == first) {
		return TRUE;
	}
	
	this->did_change = TRUE;
	this->current    = first;
	this->is_valid   = TRUE;
	
	return TRUE;
}

Bool
OPDictIterator::Last ()
{
	OPDictRef        dict = this->source;
	OPDict::Element* last = dict->tail;
	
	if (this->current == last) {
		return TRUE;
	}
	
	this->did_change = TRUE;
	this->current    = last;
	this->is_valid   = TRUE;
	
	return TRUE;
}

Bool
OPDictIterator::Next (Count move)
{
	if ( (this->current)
	  && (this->current->next) ) {
		this->current    = this->current->next;
		this->did_change = TRUE;
		this->is_valid   = TRUE;
		return TRUE;
	}

	this->is_valid = FALSE;
	
	return FALSE;
}

Bool
OPDictIterator::Prev (Count move)
{
	if ( (this->current)
	  && (this->current->prev) ) {
		this->current    = this->current->prev;
		this->did_change = TRUE;
		this->is_valid   = TRUE;
		return TRUE;
	}
	
	this->is_valid = FALSE;
	
	return FALSE;
}

Bool
OPDictIterator::Get (OPObjectOut object, Count index)
{
	OP_METHOD (Get, 2);
	OP_ARG (1, OPObjectOut, object);
	OP_ARG (2, Count, index);
	
	return this->GetValue (object, index);
}


Bool
OPDictIterator::GetKey (OPObjectOut object, Count index)
{
	OP_METHOD (GetKey, 2);
	OP_ARG (1, OPObjectOut, object);
	OP_ARG (2, Count, index);
	
	this->did_change = FALSE;
	
	if ( (index == 0)
	  && (this->current)
	  && (this->is_valid) ) {
		object = this->current->key;
		return TRUE;
	}
	
	return FALSE;
}
Bool
OPDictIterator::GetValue (OPObjectOut object, Count index)
{
	OP_METHOD (GetValue, 2);
	OP_ARG (1, OPObjectOut, object);
	OP_ARG (2, Count, index);
	
	this->did_change = FALSE;
	
	if ( (index == 0)
	  && (this->current)
	  && (this->is_valid) ) {
		object = this->current->value;
		return TRUE;
	}
	
	return FALSE;
}
