/*
 *	lookmanicon.cxx
 *
 *	Look Manager - icon handling. See lookman.cxx.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/grail.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>
#include <OPaC/values.h>

OP_IMPLEMENTATION_FILE;

/****************************************************************/

struct OPLookMan::UserIconDef
{
	UserIconDef*		next;
	UserIconDef*		prev;
	OPLookIcon			icon;
	OPAtomRef			name;
	OPGrailPixmapRef	pixmap;
	OPRect				box;
};

OPLookMan::UserIconDef*	OPLookMan::user_icon_head = 0;
OPLookMan::UserIconDef*	OPLookMan::user_icon_tail = 0;
Count					OPLookMan::user_icon_num  = 0;

/****************************************************************/

#define	DEF_ENUM_BEGIN(T)	OP_IMPLEMENT_TYPEENUM(T) {
#define	DEF_ENUM(T,n)		OP_IMPLEMENT_TYPEENUM_DEF(T)
#define	DEF_ENUM_END(T)		OP_IMPLEMENT_TYPEENUM_END() };

#include <OPaC/enum/lookicon.h>

/****************************************************************/

OPLookIcon OPLookMan::icon_conv[OP_ICON_NUM] =
{
	OP_ICON_UP_NORMAL		,
	OP_ICON_UP_PRESS		,
	OP_ICON_UP_HILITE		,
	OP_ICON_UP_DIM			,
	
	OP_ICON_DOWN_NORMAL		,
	OP_ICON_DOWN_PRESS		,
	OP_ICON_DOWN_HILITE		,
	OP_ICON_DOWN_DIM		,
	
	OP_ICON_LEFT_NORMAL		,
	OP_ICON_LEFT_PRESS		,
	OP_ICON_LEFT_HILITE		,
	OP_ICON_LEFT_DIM		,
	
	OP_ICON_RIGHT_NORMAL	,
	OP_ICON_RIGHT_PRESS		,
	OP_ICON_RIGHT_HILITE	,
	OP_ICON_RIGHT_DIM		,
	
	OP_ICON_ACCEPT_NORMAL	,
	OP_ICON_ACCEPT_PRESS	,
	OP_ICON_ACCEPT_HILITE	,
	OP_ICON_ACCEPT_DIM		,
	
	OP_ICON_CANCEL_NORMAL	,
	OP_ICON_CANCEL_PRESS	,
	OP_ICON_CANCEL_HILITE	,
	OP_ICON_CANCEL_DIM		,
	
	OP_ICON_RADIO_0_NORMAL	,
	OP_ICON_RADIO_1_NORMAL	,
	OP_ICON_RADIO_0_PRESS	,
	OP_ICON_RADIO_1_PRESS	,
	OP_ICON_RADIO_0_HILITE	,
	OP_ICON_RADIO_1_HILITE	,
	OP_ICON_RADIO_0_DIM		,
	OP_ICON_RADIO_1_DIM		,
	
	OP_ICON_CHECK_0_NORMAL	,
	OP_ICON_CHECK_1_NORMAL	,
	OP_ICON_CHECK_0_PRESS	,
	OP_ICON_CHECK_1_PRESS	,
	OP_ICON_CHECK_0_HILITE	,
	OP_ICON_CHECK_1_HILITE	,
	OP_ICON_CHECK_0_DIM		,
	OP_ICON_CHECK_1_DIM		,
	
	OP_ICON_I_BEAM			,
	OP_ICON_I_BEAM_MASK		,
	
	OP_ICON_GREEN_CLED_OFF	,
	OP_ICON_GREEN_CLED_ON	,
	OP_ICON_GREEN_CLED_HALF	,
	
	OP_ICON_RED_CLED_OFF	,
	OP_ICON_RED_CLED_ON		,
	OP_ICON_RED_CLED_HALF	,
	
	OP_ICON_GREEN_SLED_OFF	,
	OP_ICON_GREEN_SLED_ON	,
	OP_ICON_GREEN_SLED_HALF	,
	
	OP_ICON_RED_SLED_OFF	,
	OP_ICON_RED_SLED_ON		,
	OP_ICON_RED_SLED_HALF	,
	
	OP_ICON_H_SLIDER_LEFT	,
	OP_ICON_H_SLIDER_RIGHT	,
	OP_ICON_H_SLIDER_SMALL	,
	
	OP_ICON_V_SLIDER_TOP	,
	OP_ICON_V_SLIDER_BOT	,
	OP_ICON_V_SLIDER_SMALL	,
	
	OP_ICON_SUBMENU_NORMAL	,
	OP_ICON_SUBMENU_PRESS	,
	OP_ICON_SUBMENU_HILITE	,
	OP_ICON_SUBMENU_DIM		,
	
	OP_ICON_PULLDOWN_NORMAL	,
	OP_ICON_PULLDOWN_PRESS	,
	OP_ICON_PULLDOWN_HILITE	,
	OP_ICON_PULLDOWN_DIM	,
	
	OP_ICON_POPUP_NORMAL	,
	OP_ICON_POPUP_PRESS		,
	OP_ICON_POPUP_HILITE	,
	OP_ICON_POPUP_DIM		,
	
	OP_ICON_DOWN_ARROW		,
	OP_ICON_UP_ARROW		,
	
	OP_ICON_TREE_BRANCH		,
	OP_ICON_TREE_TRUNK		,
	OP_ICON_TREE_BOTTOM		,
	OP_ICON_TREE_BOX		,
	OP_ICON_TREE_BOX_BOT	,
	OP_ICON_TREE_MINUS		,
	OP_ICON_TREE_MINUS_BOT	,
	OP_ICON_TREE_PLUS		,
	OP_ICON_TREE_PLUS_BOT	,
	OP_ICON_TREE_HOME		,
	OP_ICON_TREE_DIR		,
	OP_ICON_TREE_DIR_OPEN	,
	OP_ICON_TREE_FILE		,

	OP_ICON_A_PLUS_NORMAL	,
	OP_ICON_A_PLUS_PRESS	,
	OP_ICON_A_PLUS_HILITE	,
	OP_ICON_A_PLUS_DIM		,
	
	OP_ICON_A_MINUS_NORMAL	,
	OP_ICON_A_MINUS_PRESS	,
	OP_ICON_A_MINUS_HILITE	,
	OP_ICON_A_MINUS_DIM		,
	
	OP_ICON_MINCLOSE_NORMAL	,
	OP_ICON_MINCLOSE_PRESS	,
	OP_ICON_MINCLOSE_HILITE	,
	OP_ICON_MINCLOSE_DIM	,
	
	OP_ICON_SLIDOT_NORMAL	,
	OP_ICON_SLIDOT_PRESS	,
	OP_ICON_SLIDOT_HILITE	,
	OP_ICON_SLIDOT_DIM		,
	
	OP_ICON_CUBE_NORMAL		,
	OP_ICON_CUBE_NULL		,
	OP_ICON_CUBE_HILITE		,
	OP_ICON_CUBE_ACTIVE		,
	
	OP_ICON_TRASH_NORMAL	,
	OP_ICON_TRASH_EMPTY		,
	OP_ICON_TRASH_HILITE	,
	OP_ICON_TRASH_ACTIVE	,
	
	OP_ICON_RIGHT_ARROW		,
	OP_ICON_LEFT_ARROW		,
	
	OP_ICON_OBJ_APP			,
	OP_ICON_OBJ_WINDOW		,
	OP_ICON_OBJ_MENU		,
	OP_ICON_OBJ_SLOT		
};

/****************************************************************/

OPAtomRef OPLookMan::icon_name[OP_ICON_NUM];

static const char* icon_name_s[OP_ICON_NUM] =
{
	"OP_ICON_UP_NORMAL"		,
	"OP_ICON_UP_PRESS"		,
	"OP_ICON_UP_HILITE"		,
	"OP_ICON_UP_DIM"		,
	
	"OP_ICON_DOWN_NORMAL"	,
	"OP_ICON_DOWN_PRESS"	,
	"OP_ICON_DOWN_HILITE"	,
	"OP_ICON_DOWN_DIM"		,
	
	"OP_ICON_LEFT_NORMAL"	,
	"OP_ICON_LEFT_PRESS"	,
	"OP_ICON_LEFT_HILITE"	,
	"OP_ICON_LEFT_DIM"		,
	
	"OP_ICON_RIGHT_NORMAL"	,
	"OP_ICON_RIGHT_PRESS"	,
	"OP_ICON_RIGHT_HILITE"	,
	"OP_ICON_RIGHT_DIM"		,
	
	"OP_ICON_ACCEPT_NORMAL"	,
	"OP_ICON_ACCEPT_PRESS"	,
	"OP_ICON_ACCEPT_HILITE"	,
	"OP_ICON_ACCEPT_DIM"	,
	
	"OP_ICON_CANCEL_NORMAL"	,
	"OP_ICON_CANCEL_PRESS"	,
	"OP_ICON_CANCEL_HILITE"	,
	"OP_ICON_CANCEL_DIM"	,
	
	"OP_ICON_RADIO_0_NORMAL",
	"OP_ICON_RADIO_1_NORMAL",
	"OP_ICON_RADIO_0_PRESS"	,
	"OP_ICON_RADIO_1_PRESS"	,
	"OP_ICON_RADIO_0_HILITE",
	"OP_ICON_RADIO_1_HILITE",
	"OP_ICON_RADIO_0_DIM"	,
	"OP_ICON_RADIO_1_DIM"	,
	
	"OP_ICON_CHECK_0_NORMAL",
	"OP_ICON_CHECK_1_NORMAL",
	"OP_ICON_CHECK_0_PRESS"	,
	"OP_ICON_CHECK_1_PRESS"	,
	"OP_ICON_CHECK_0_HILITE",
	"OP_ICON_CHECK_1_HILITE",
	"OP_ICON_CHECK_0_DIM"	,
	"OP_ICON_CHECK_1_DIM"	,
	
	"OP_ICON_I_BEAM"		,
	"OP_ICON_I_BEAM_MASK"	,
	
	"OP_ICON_GREEN_CLED_OFF",
	"OP_ICON_GREEN_CLED_ON"	,
	"OP_ICON_GREEN_CLED_HALF",
	
	"OP_ICON_RED_CLED_OFF"	,
	"OP_ICON_RED_CLED_ON"	,
	"OP_ICON_RED_CLED_HALF"	,
	
	"OP_ICON_GREEN_SLED_OFF",
	"OP_ICON_GREEN_SLED_ON"	,
	"OP_ICON_GREEN_SLED_HALF",
	
	"OP_ICON_RED_SLED_OFF"	,
	"OP_ICON_RED_SLED_ON"	,
	"OP_ICON_RED_SLED_HALF"	,
	
	"OP_ICON_H_SLIDER_LEFT"	,
	"OP_ICON_H_SLIDER_RIGHT",
	"OP_ICON_H_SLIDER_SMALL",
	
	"OP_ICON_V_SLIDER_TOP"	,
	"OP_ICON_V_SLIDER_BOT"	,
	"OP_ICON_V_SLIDER_SMALL",
	
	"OP_ICON_SUBMENU_NORMAL",
	"OP_ICON_SUBMENU_PRESS"	,
	"OP_ICON_SUBMENU_HILITE",
	"OP_ICON_SUBMENU_DIM"	,
	
	"OP_ICON_PULLDOWN_NORMAL",
	"OP_ICON_PULLDOWN_PRESS",
	"OP_ICON_PULLDOWN_HILITE",
	"OP_ICON_PULLDOWN_DIM"	,
	
	"OP_ICON_POPUP_NORMAL"	,
	"OP_ICON_POPUP_PRESS"	,
	"OP_ICON_POPUP_HILITE"	,
	"OP_ICON_POPUP_DIM"		,
	
	"OP_ICON_DOWN_ARROW"	,
	"OP_ICON_UP_ARROW"		,
	
	"OP_ICON_TREE_BRANCH"	,
	"OP_ICON_TREE_TRUNK"	,
	"OP_ICON_TREE_BOTTOM"	,
	"OP_ICON_TREE_BOX"		,
	"OP_ICON_TREE_BOX_BOT"	,
	"OP_ICON_TREE_MINUS"	,
	"OP_ICON_TREE_MINUS_BOT",
	"OP_ICON_TREE_PLUS"		,
	"OP_ICON_TREE_PLUS_BOT"	,
	"OP_ICON_TREE_HOME"		,
	"OP_ICON_TREE_DIR"		,
	"OP_ICON_TREE_DIR_OPEN"	,
	"OP_ICON_TREE_FILE"		,
	
	"OP_ICON_A_PLUS_NORMAL"	,
	"OP_ICON_A_PLUS_PRESS"	,
	"OP_ICON_A_PLUS_HILITE"	,
	"OP_ICON_A_PLUS_DIM"	,
	
	"OP_ICON_A_MINUS_NORMAL",
	"OP_ICON_A_MINUS_PRESS"	,
	"OP_ICON_A_MINUS_HILITE",
	"OP_ICON_A_MINUS_DIM"	,
	
	"OP_ICON_MINCLOSE_NORMAL",
	"OP_ICON_MINCLOSE_PRESS",
	"OP_ICON_MINCLOSE_HILITE",
	"OP_ICON_MINCLOSE_DIM"	,
	
	"OP_ICON_SLIDOT_NORMAL"	,
	"OP_ICON_SLIDOT_PRESS"	,
	"OP_ICON_SLIDOT_HILITE"	,
	"OP_ICON_SLIDOT_DIM"	,
	
	"OP_ICON_CUBE_NORMAL"	,
	"OP_ICON_CUBE_NULL"		,
	"OP_ICON_CUBE_HILITE"	,
	"OP_ICON_CUBE_ACTIVE"	,
	
	"OP_ICON_TRASH_NORMAL"	,
	"OP_ICON_TRASH_EMPTY"	,
	"OP_ICON_TRASH_HILITE"	,
	"OP_ICON_TRASH_ACTIVE"	,
	
	"OP_ICON_RIGHT_ARROW"	,
	"OP_ICON_LEFT_ARROW"	,

	"OP_ICON_OBJ_APP"		,
	"OP_ICON_OBJ_WINDOW"	,
	"OP_ICON_OBJ_MENU"		,
	"OP_ICON_OBJ_SLOT"		
};

/****************************************************************/

void
OPLookMan::InitIcons ()
{
	OP_METHOD (InitIcons, 0);
	
	Count i;
	
	this->icon_cache             = OP_ICON_NONE;
	this->icon_cache_index       = -1;
	
	for (i = 0; i < OP_ICON_NUM; i++) {
		this->icon_name[i] = icon_name_s[i];
	}
	
	this->sys_icons = OPaC_LoadGIF ("%OPaC%/op_icons.gif", OPaC_sys_zone);
	
	OP_ASSERT (this->sys_icons.IsValid ());
	
	//	Two special color values are used in the GIF file : the transparent
	//	color and the separator color, used to find the size of an icon.
	
	OPColorPixel trans = this->sys_icons->ReturnPixel (0, 0);
	OPColorPixel sep   = this->sys_icons->ReturnPixel (0, 1);
	OPColorPixel alpha = OPColorPixel (1.0, 1.0, 1.0, 1.0);
	
	OPCoord sys_i_dx = 0;
	OPCoord sys_i_dy = 0;
	OPCoord sys_x  = 1;
	OPCoord sys_y  = 1;
	OPCoord sys_dx = 0;
	OPCoord sys_dy = 0;
	
	this->sys_icons->GetSize (sys_i_dx, sys_i_dy);
	
	for (i = 0; i < OP_ICON_NUM; i++) {
		
		OPColorPixel pixel = 0;
		
		if (sys_x == 1) {
			
			//	Find out about the height of the current band.
			
			sys_y += sys_dy;
			sys_dy = 0;
			
			for (;;) {
				
				OP_ASSERT (sys_y < sys_i_dy);
				
				pixel = this->sys_icons->ReturnPixel (0, sys_y);
				
				if (pixel == sep) {
					break;
				}
				
				sys_y++;
			}

			//	We have just found the vertical position of the band of icons. We
			//	will look for the height now.
			
			for (;;) {
				
				OP_ASSERT (sys_y+sys_dy < sys_i_dy);
				
				pixel = this->sys_icons->ReturnPixel (0, sys_y+sys_dy++);
				
				if (pixel == trans) {
					break;
				}
			}
		}
		
		sys_dx = 1;
		
		for (;;) {
			
			OP_ASSERT (sys_y < sys_i_dy);
			OP_ASSERT (sys_x+sys_dx < sys_i_dx);
			
			//	Find the width of the current icon. Marks at the top of them
			//	signal when a new icon starts.
			
			pixel = this->sys_icons->ReturnPixel (sys_x+sys_dx, sys_y-1);
			
			if (pixel == sep) {
				break;
			}
			
			sys_dx++;
		}
		
		this->icon_ox[i] = sys_x;
		this->icon_oy[i] = sys_y;
		this->icon_dx[i] = sys_dx;
		this->icon_dy[i] = sys_dy;
		
		//	Look if this icon contains transparent pixels. If so, replace them
		//	with really transparent (alpha = 1) pixels.
		
		OPCoord iy  = sys_y;
		OPCoord idy = sys_dy;
		Bool    itr = FALSE;
		
		for (; (idy > 0) && (!itr); idy--, iy++) {
			
			OPCoord ix  = sys_x;
			OPCoord idx = sys_dx;
			
			for (; idx > 0; idx--, ix++) {
				
				pixel = this->sys_icons->ReturnPixel (ix, iy);
				
				if (pixel == trans) {
					this->sys_icons->DrawDot (ix, iy, alpha);
				}
			}
		}
		
		sys_x += sys_dx;
		
		if ( (i < OP_ICON_NUM-1)
		  && ( (OPLookMan::icon_conv[i] / 0x1000)
			!= (OPLookMan::icon_conv[i+1] / 0x1000) ) ) {
			
			//	Skip to the next line. The next icon will be at the very left
			//	of the next band of icons.
			
			sys_x = 1;
		}
	}
	
}

/****************************************************************/

static int
find_icon (OPLookIcon look, const OPLookIcon* table)
{
	for (Count icon = 0; icon < OP_ICON_NUM; icon++) {
		if (*table++ == look) {
			return icon;
		}
	}

	return -1;
}


Bool
OPLookMan::GetIconSize (OPLookIcon icon, OPCoordOut x, OPCoordOut y) const
{
	OP_METHOD (GetIconSize, 3);
	OP_ARG (1, OPLookIcon, icon);
	OP_ARG (2, OPCoordOut, x);
	OP_ARG (3, OPCoordOut, y);
	
	if (this->UpdateIconCache (icon)) {
		x = this->icon_cache_box.DX ();
		y = this->icon_cache_box.DY ();
		return TRUE;
	}
	
	x = 0;
	y = 0;
	
	return FALSE;
}

Bool
OPLookMan::GetIconPos (OPLookIcon icon, OPCoordOut x, OPCoordOut y) const
{
	OP_METHOD (GetIconSize, 3);
	OP_ARG (1, OPLookIcon, icon);
	OP_ARG (2, OPCoordOut, x);
	OP_ARG (3, OPCoordOut, y);
	
	if (this->UpdateIconCache (icon)) {
		x = this->icon_cache_box.X1 ();
		y = this->icon_cache_box.Y1 ();
		return TRUE;
	}
	
	x = 0;
	y = 0;
	
	return FALSE;
}

Bool
OPLookMan::GetIconName (OPLookIcon icon, OPAtomOut name) const
{
	OP_METHOD (GetIconName, 2);
	OP_ARG (1, OPLookIcon, icon);
	OP_ARG (2, OPAtomOut, name);
	
	if (this->UpdateIconCache (icon)) {
		name = this->icon_name[this->icon_cache_index];
		return TRUE;
	}
	
	name = "";
	
	return FALSE;
}



Bool
OPLookMan::UpdateIconCache (OPLookIcon icon) const
{
	OP_METHOD (UpdateIconCache, 1);
	OP_ARG (1, OPLookIcon, icon);
	
	if (icon == this->icon_cache) {
		return TRUE;
	}
	
	int index = find_icon (icon, OPLookMan::icon_conv);
	
	if (index == -1) {
		UserIconDef* def = OPLookMan::user_icon_head;
		
		while (def) {
			if (def->icon == icon) {
				this->UpdateIconCache (icon, def);
				return TRUE;
			}
			def = def->next;
		}
		
		return FALSE;
	}
	
	this->UpdateIconCache (icon, index);
	return TRUE;
}


/*
 *	Return the nth icon, according to its index.
 */

OPLookIcon
OPLookMan::FindNthIcon (Count index)
{
	OP_METHOD (FindNthIcon, 1);
	OP_ARG (1, Count, index);
	
	if (index < OP_ICON_NUM) {
		return OPLookMan::icon_conv[index];
	}
	
	index -= OP_ICON_NUM;
	UserIconDef* def = OPLookMan::user_icon_head;
	
	while (def && index) {
		def = def->next;
		index--;
	}
	
	return ( (index) || (def == 0) ) ? OP_ICON_NONE : def->icon;
}


void
OPLookMan::UpdateIconCache (OPLookIcon icon, int index) const
{
	this->icon_cache        = icon;
	this->icon_cache_index  = index;
	this->icon_cache_pixmap = this->ReturnSysIcons ();
	this->icon_cache_box.SetOrigin (this->icon_ox[index], this->icon_oy[index]);
	this->icon_cache_box.SetSize (this->icon_dx[index], this->icon_dy[index]);
}

void
OPLookMan::UpdateIconCache (OPLookIcon icon, const UserIconDef* def) const
{
	this->icon_cache        = icon;
	this->icon_cache_index  = -1;
	this->icon_cache_pixmap = def->pixmap;
	this->icon_cache_box    = def->box;
}

const OPGrailPixmapRef&
OPLookMan::ReturnIconPixmap (OPLookIcon icon)
{
	OP_METHOD (ReturnIconPixmap, 1);
	OP_ARG (1, OPLookIcon, icon);
	
	if (this->UpdateIconCache (icon)) {
		return this->icon_cache_pixmap;
	}
	
	return this->sys_icons;
}


OPLookIcon
OPLookMan::FindIcon (OPAtomIn name)
{
	OP_METHOD (FindIcon, 1);
	OP_ARG (1, OPAtomIn, name);
	
	for (Count i = 0; i < OP_ICON_NUM; i++) {
		if (this->icon_name[i] == name) {
			return this->icon_conv[i];
		}
	}
	
	UserIconDef* icon = OPLookMan::user_icon_head;
	
	while (icon) {
		if (icon->name == name) {
			return icon->icon;
		}
		icon = icon->next;
	}
	
	return OP_ICON_NONE;
}

/******************************************************************************/

void
OPLookMan::Insert (UserIconDef* icon)
{
	icon->next = OPLookMan::user_icon_head;
	icon->prev = 0;
	(icon->next ? icon->next->prev : OPLookMan::user_icon_tail) = icon;
	(icon->prev ? icon->prev->next : OPLookMan::user_icon_head) = icon;
}

void
OPLookMan::Remove (UserIconDef* icon)
{
	(icon->next ? icon->next->prev : OPLookMan::user_icon_tail) = icon->prev;
	(icon->prev ? icon->prev->next : OPLookMan::user_icon_head) = icon->next;
}

/******************************************************************************/

OPLookIcon
OPLookMan::AddUserIcon (OPAtomIn name, OPGrailPixmapIn src, OPRectIn box)
{
	OP_METHOD (AddUserIcon, 3);
	OP_ARG (1, OPAtomIn, name);
	OP_ARG (2, OPGrailPixmapIn, src);
	OP_ARG (3, OPRectIn, box);
	
	OPLookIcon found = OP_ICON_NONE;
	
	if (name.IsValid ()) {
		found = this->FindIcon (name);
	}
	
	if (found == OP_ICON_NONE) {
		
		UserIconDef* icon = NEW UserIconDef;
		found = (OPLookIcon)(OP_ICON_USER + OPLookMan::user_icon_num++);
		
		icon->icon              = found;
		icon->name              = name;
		icon->pixmap            = src;
		icon->box               = box;
		
		this->Insert (icon);
	}
	
	return found;
}

void
OPLookMan::RemoveUserIcon (OPLookIcon icon)
{
	OP_METHOD (RemoveUserIcon, 1);
	OP_ARG (1, OPLookIcon, icon);
	
	UserIconDef* iter = OPLookMan::user_icon_head;
	
	while (iter) {
		if (iter->icon == icon) {
			this->Remove (iter);
			DELETE (iter);
			return;
		}
		iter = iter->next;
	}
	
}

/******************************************************************************/
