/*
 *	maininit.cxx
 *
 *	System independent main initialisation. This is called by
 *	the application's "main" (or an equivalent).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <stdio.h>
#include <OPaC/app.h>
#include <OPaC/except.h>
#include <OPaC/grail.h>
#include <OPaC/lookman.h>
#include <OPaC/new.h>

OP_IMPLEMENTATION_FILE;

extern void OPaC_MainInit ();


void
OPaC_MainInit ()
{
	OP_FUNCTION (OPaC_MainInit, 0);
	
#ifdef _DEBUG
	remove ("leak-mem.log");
	remove ("dead-ref.log");
#endif
	
	try {
		OPMemZone::Initialise ();
		OPAtom::Initialise ();
		OPClassMan::Initialise ();
		
		{
			//	Setup color and font managers.

			OPFontManRef  font_man  = OP_NEW (OPFontMan);
			
			OPAppRef app = (OPAppRef) OPClassMan::New ("App", OPaC_app_zone, __LINE__, __OPaC_src_file);
			OPApp::Initialise (app);
			
			OPFontMan::InitSystemFontMan (font_man);
			OPLookMan::Initialise ();
			
			//	Setup application object.

			OPLookMan::Current ()->SelectCursor (OP_CURSOR_NORMAL);
			
			app->Initialise ();
			app->MainEventLoop ();

			OPApp::Kill ();
			OPLookMan::Kill ();
			OPFontMan::KillSystemFontMan ();
			
			font_man  = 0;
		}
		
#ifdef _DEBUG
		if (OPObjectRef::g_head) {
			OPObjectRef* iter = OPObjectRef::g_head;
			FILE* file = 0;
			
			while (iter) {
				if ( (iter->ReturnEmbeddingObject () == 0)
				  && (iter->ReturnEmbeddingFrame ()) ) {
					
					if (file == 0) {
						file = fopen ("dead-ref.log", "wt");
						if (file == 0) break;
					}
					
					const OPaC_CallFrame* info = iter->ReturnEmbeddingFrame ();
					
					fprintf (file, "%sRef: %s:%ld in %s '%s'\n",
								   iter->ReturnClassName (),
								   info->ReturnFileName (),
								   info->ReturnFileLine (),
								   OPaC_CallFrame::ReturnTypeName (info->ReturnType ()),
								   info->ReturnFuncName ());
				}
				iter = iter->g_next;
			}
			
			if (file) fclose (file);
		}
#endif
	}
	
	catch (OPExcMemory e_memory) {
		
		char buffer[500];
		snprintf (buffer, sizeof (buffer),
				  "\nFatal memory exception raised by caller %s:%d.",
				  e_memory.ReturnFaultySrcFile (),
				  e_memory.ReturnFaultySrcLine ());
		
		OPaC_ErrorPrint (buffer,
						 e_memory.ReturnThrowSrcLine (),
						 e_memory.ReturnThrowSrcFile ());
	}
	
	catch (OPException e_any) {
		
		OPaC_ErrorPrint ("\nFatal OPaC exception raised...",
						 e_any.ReturnThrowSrcLine (),
						 e_any.ReturnThrowSrcFile ());
	}
	
	catch (...) {
		
		OPaC_ErrorPrint ("\nFatal system exception raised...", 0, "?");
	}
	
	OPClassMan::Kill ();
	OPAtom::Kill ();
	OPMemZone::Kill ();
}


