/*
 *	monitorvalue.cxx
 *
 *	Monitor one (or several) value(s) and display them in a
 *	view (e.g. a gauge, a scope, etc.)
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/array.h>
#include <OPaC/monitorvalue.h>
#include <OPaC/new.h>
#include <OPaC/pubmacro.h>
#include <OPaC/targetmacro.h>
#include <OPaC/values.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPMonitorValue);

/****************************************************************/
	
OP_TARGET_ACTOR (OPMonitorValue, FlushCache);
OP_TARGET_ACTOR (OPMonitorValue, FlushCacheAndRedraw);

/****************************************************************/

OP_PUBLISH_SLOT (OPMonitorValue, value, ReturnValue, SetValue, "value", OPValue);

/****************************************************************/

/* 
 *	
 */

OPMonitorValue::OPMonitorValue ()
{
	OP_METHOD (OPMonitorValue, 0);
}



/*
 *	
 */

OPMonitorValue::~OPMonitorValue ()
{
	OP_METHOD (~OPMonitorValue, 0);
}



/*
 *
 */

void
OPMonitorValue::Delete ()
{
	OP_METHOD (Delete, 0);
	
	OPxSmartSource::Cleanup ();
	super::Delete ();
}

/*
 *	Save 
 */

void
OPMonitorValue::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	OPxSmartSource::StoreInstance (storage);
	
	static OPStorageSync info ("<OPMonitorValue>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << OPStorageSync::end;
}



/*
 *	Restore 
 */

void
OPMonitorValue::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	OPxSmartSource::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> OPStorageSync::end;
}



/*
 *	Either change the value of the data source or define
 *	a new value just for this monitor.
 */

Bool
OPMonitorValue::SetValue (OPValueIn value)
{
	OP_METHOD (SetValue, 1);
	OP_ARG (1, OPValueIn, value);
	
	if (this->num == 0) {
		
		this->num = 1;
		this->is_cache_ok = FALSE;
		
		this->query_roots->SetSize (this->num);
		this->query_names->SetSize (this->num);
		
		this->query_roots[0] = OPObjectNull;
		this->query_names[0] = OPAtomNull;
	}
	
	this->value = value;
	
	return this->SetNthValue (0, value);
}

OPValueRef
OPMonitorValue::ReturnValue () const
{
	OP_METHOD (ReturnValue, 0);
	
	if (this->GetNthValue (0, this->value) == FALSE) {
		this->value = OPValueNull;
	}
	
	return this->value;
}


/*
 *	Flush the data cache and redraw the view.
 */

void
OPMonitorValue::FlushCacheAndRedraw ()
{
	OP_METHOD (FlushCacheAndRedraw, 0);
	
	this->FlushCache ();
	this->Display ();
}



void
OPMonitorValue::Broadcast (OPAtomIn message, OPObjectIn value)
{
	OP_METHOD (Broadcast, 2);
	OP_ARG (1, OPAtomIn, message);
	OP_ARG (2, OPObjectIn, value);
	
	static OPAtomRef m_refresh = "Refresh";
	
	super::Broadcast (message, value);
	
	if (message == m_refresh) {
		this->FlushCache ();
	}
}

void
OPMonitorValue::Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what)
{
	OP_METHOD (Draw, 3);
	OP_ARG (1, OPGrailPortIn, port);
	OP_ARG (2, OPRectIn, rect);
	OP_ARG (3, OPDrawOp, what);
	
	this->UpdateInternal ();
	
	super::Draw (port, rect, what);
}

