/*
 *	conbrowsedlg.cxx
 *
 *	Fill in a view with the information needed to establish the
 *	connections between an event and an action.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "buildedit.h"
#include "connectdlg.h"

#include <OPaC/button.h>
#include <OPaC/database.h>
#include <OPaC/datarep.h>
#include <OPaC/icon.h>
#include <OPaC/new.h>
#include <OPaC/scroller.h>
#include <OPaC/selgrid.h>
#include <OPaC/string.h>

#include <OPaC/targetmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPConBrowseDialog);

OP_TARGET_OBSERVER	(OPConBrowseDialog, IconsChanged);

/****************************************************************/

OPConBrowseDialog::OPConBrowseDialog ()
{
	OP_METHOD (OPConBrowseDialog, 0);
	this->SetAttribute ("OPaC.volatile.observer");
}

OPConBrowseDialog::~OPConBrowseDialog ()
{
	OP_METHOD (~OPConBrowseDialog, 0);
}


/****************************************************************/

void
OPConBrowseDialog::Initialise (OPObjectIn src, OPObjectIn dst)
{
	OP_METHOD (Initialise, 2);
	OP_ARG (1, OPObjectIn, src);
	OP_ARG (2, OPObjectIn, dst);
	
	OPButtonRef button;
	
	OPCoord x  = 0;
	OPCoord y  = 0;
	OPCoord dx = 0;
	OPCoord dy = 0;
	
	this->GetOrigin (x, y);
	this->GetSize (dx, dy);
	
	dx -= 20;
	dy -= 10;
	x  += 5;
	y  += 5;
	dx /= 3;
	
	this->source = src;
	this->target = dst;
	
	this->CreateSourceList (x, y, dx, dy);
	
	OPObjRootProviderRef root = OP_NEW (OPObjRootProvider);
	root->Initialise (this->target);
	
	this->target_browser = OP_NEW (OPObjectBrowser);
	this->target_browser->SetParent (this);
	this->target_browser->SetOrigin (x+dx+5, y+50);
	this->target_browser->SetSize (dx*2, dy-50-20);
	this->target_browser->Initialise (2, root, "OPSelGrid");
	this->target_browser->AttachObserver (this, "IconsChanged");
	
	OPScrollerRef   scroller = OP_NEW (OPScroller);
	OPDataReportRef datarep1 = OP_NEW (OPDataReport);
	OPDataReportRef datarep2 = OP_NEW (OPDataReport);
	OPDataReportRef datarep3 = OP_NEW (OPDataReport);
	
	scroller->SetParent (this);
	scroller->Initialise (x+dx+5, y+dy-20+3, dx*2, TRUE);
	
	datarep1->Initialise (this->target_browser, "Changed", "visible_value", scroller, "visible_value");
	datarep2->Initialise (this->target_browser, "Changed", "visible_ratio", scroller, "visible_ratio");
	datarep3->Initialise (scroller, "Changed", "visible_value", this->target_browser, "visible_value");
}

/****************************************************************/

/*
 *	This call-back method is invoked when the user presses
 *	on the "Connect" button. It will establish a connection
 *	(or several, if possible).
 */

void
OPConBrowseDialog::Connect ()
{
	OP_METHOD (Connect, 0);
	
	Count nth = 0;
	
	if (this->source_list->CountSelectedItems ()) {
		
		OPAtomRef   src;
		OPAtomRef   notifier;
		OPTextRef   text;
		OPStringRef path;
		
		this->source_list->GetSelectedName (src);
		this->source_list->GetSelectedText (text);
		this->target_browser->GetSelectedPath (path);
		
		text->GetAttribute ("Notifier", notifier);
		
		Count pos = 0;
		OPStringRef sub_path;
		
		//	The path will look like "Object.x.[3].y.z". We have to strip the
		//	leading "Object" since we know it directly.
		
		if (path->Find (".", pos)) {
			
			path->MakeMiddle (pos+1, 1000, sub_path);
			
			OPStringRef     action;
			OPDataReportRef report = OP_NEW (OPDataReport);
			
			pos = sub_path->CountCharacters ()-1;
			
			//	The path has now been stripped from its leading indirection.
			//	If the target object is a view, define a "Display" action for
			//	it (if possible).
			
			if ( (this->use_display_actor)
			  && (sub_path->Find (".", pos, -1))
			  && (this->target->IsKindOf (OP_RUNTIME (OPView))) ) {
				
				sub_path->MakeLeft (pos, action);
				
				action->AppendString (".Display");
				report->DefineActionPath (action);
			}
			
			//	Attach the reporter to the source (which will be queried) and
			//	to the destination (which will be set). The source object is
			//	not defined explicitely but report's observer will get it...
			
			report->DefineQuery (src);
			report->DefineTargetPath (this->target, sub_path);
			
			this->source->AttachObserver (report, notifier, "Changed");
		}
	}
}



/*
 *	The browse dialog's icons have changed (either a pane
 *	has been opened/closed or the active pane has changed).
 */

void
OPConBrowseDialog::IconsChanged (OPObjectIn object, Card32 id)
{
	OP_METHOD (IconsChanged, 2);
	OP_ARG (1, OPObjectIn, object);
	OP_ARG (2, Card32, id);
	
	OPObjectBrowserRef browser = object;
	Count index = browser->ReturnZeroPane ();
	OPIconRef icon;
	OPCoord   x, y;
	this->GetOrigin (x, y);
	
	while (browser->GetIcon (index++, icon, x)) {
		icon->SetParent (this);
		icon->SetOrigin (x, y);
	}
	
	this->Display ();
}
