/*
 *	connectdlg.cxx
 *
 *	Fill in a window with the information needed to establish the
 *	connections between a source and a target. This will show a
 *	tabbed view with two tabs :
 *
 *	- Source event and target action.
 *	- Source data field and target data field.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "connectdlg.h"

#include <OPaC/button.h>
#include <OPaC/container.h>
#include <OPaC/new.h>
#include <OPaC/tablist.h>
#include <OPaC/tabview.h>
#include <OPaC/texttag.h>
#include <OPaC/window.h>

#include <OPaC/notmacro.h>
#include <OPaC/targetmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPConnectDialog);

/****************************************************************/

OP_TARGET_ACTOR (OPConnectDialog, Connect);
OP_TARGET_ACTOR (OPConnectDialog, Cancel);

/****************************************************************/

OPConnectDialog::OPConnectDialog ()
{
	OP_METHOD (OPConnectDialog, 0);
	this->SetAttribute ("OPaC.volatile.observer");
}

OPConnectDialog::~OPConnectDialog ()
{
	OP_METHOD (~OPConnectDialog, 0);
}

/****************************************************************/

void
OPConnectDialog::Initialise (OPObjectIn x_src, OPObjectIn x_dst)
{
	OP_METHOD (Initialise, 2);
	OP_ARG (1, OPObjectIn, x_src);
	OP_ARG (2, OPObjectIn, x_dst);
	
	OPObjectRef obj;
	OPObjectRef src = x_src;
	OPObjectRef dst = x_dst;
	
	if (src->GetAttribute ("embedded_object_for_connect", obj)) {
		src = obj;
	}
	
	if (dst->GetAttribute ("embedded_object_for_connect", obj)) {
		dst = obj;
	}
	
	this->window = OP_NEW (OPWindow);
	this->actors = OP_NEW (OPConActorDialog);
	this->fields = OP_NEW (OPConFieldDialog);
#if 0
	this->bactor = OP_NEW (OPConBrowseDialog);
	this->bfield = OP_NEW (OPConBrowseDialog);
#endif
	
	this->window->Initialise (20, 20, 420, 280, "Connection inspector");
	this->window->SetStyle (OP_WIN_TITLE | OP_WIN_MOVE_OK);
	
	this->actors->SetParent (window);
	this->actors->SetOrigin (0, 0);
	this->actors->SetSize (400-4, 200);
	this->actors->Initialise (src, dst);
	
	this->fields->SetParent (window);
	this->fields->SetOrigin (0, 0);
	this->fields->SetSize (400-4, 200);
	this->fields->Initialise (src, dst);

#if 0
	this->bfield->SetParent (window);
	this->bfield->SetOrigin (0, 0);
	this->bfield->SetSize (400-4, 200);
	this->bfield->Initialise (src, dst);
	
	this->bactor->SetParent (window);
	this->bactor->SetOrigin (0, 0);
	this->bactor->SetSize (400-4, 200);
	this->bactor->Initialise (src, dst);
#endif
	
	OPTabListRef  tab_list = OPTabListNull;
	OPListRef     list     = OP_NEW (OPList);
	OPTextTagRef  text;
	
	this->tab_view = OP_NEW (OPTabView);
	
	this->tab_view->SetParent (window);
	this->tab_view->SetOrigin (10, 10);
	this->tab_view->SetSize (400, 230);
	
	list->SetInsertMode (FALSE);
	
	text = OP_NEW (OPTextTag);	text->SetText ("Actors");			list->Insert (text);
	text = OP_NEW (OPTextTag);	text->SetText ("Fields");			list->Insert (text);
//	text = OP_NEW (OPTextTag);	text->SetText ("Browse actors");	list->Insert (text);
//	text = OP_NEW (OPTextTag);	text->SetText ("Browse fields");	list->Insert (text);
	
	this->tab_view->GetTabs (tab_list);
	tab_list->SetTabs (list);
	
	list = OP_NEW (OPList);
	this->tab_view->SetViewList (list);
	
	list->SetInsertMode (FALSE);
	
	this->actors->SetParent (OPViewNull);
	this->fields->SetParent (OPViewNull);
//	this->bactor->SetParent (OPViewNull);
//	this->bfield->SetParent (OPViewNull);
	
	list->Insert (this->actors);
	list->Insert (this->fields);
//	list->Insert (this->bactor);
//	list->Insert (this->bfield);
	
	OPButtonRef button;
	
	button = OP_NEW (OPButton);
	button->Initialise (10, 245, 100, 25, "Connect", this->window);
	button->AttachObserver (this, "OnClick", "Connect");
	
	this->button_connect = button;
	
	button = OP_NEW (OPButton);
	button->Initialise (120, 245, 100, 25, "Cancel", this->window);
	button->AttachObserver (this, "OnClick", "Cancel");
	
	this->button_cancel = button;
	
	this->tab_view->ActivateView (1);
	this->window->Open ();
	this->window->Display ();
	this->window->ShowNormal ();
}

/*
 *	This call-back method is invoked when the user presses
 *	on the "Connect" button. It will establish a connection
 *	(or several, if possible) and close the dialog.
 */

void
OPConnectDialog::Connect ()
{
	OP_METHOD (Connect, 0);
	
	switch (this->tab_view->ReturnActiveView ()) {
		case 1:
			this->actors->Connect ();
			break;
		case 2:
			this->fields->Connect ();
			break;
#if 0
		case 3:
			this->bactor->Connect ();
			break;
		case 4:
			this->bfield->Connect ();
			break;
#endif
	}
	
	this->Kill ();
}


/*
 *	This call-back method is invoked when the user presses
 *	on the "Cancel" button. It will simply close the dialog.
 */

void
OPConnectDialog::Cancel ()
{
	OP_METHOD (Cancel, 0);
	
	this->Kill ();
}


void
OPConnectDialog::Kill ()
{
	OP_METHOD (Kill, 0);
	
	OPObjectRef self = this;
	
	this->button_connect->DetachObserver (this, "OnClick", "Connect");
	this->button_cancel->DetachObserver (this, "OnClick", "Cancel");
	
	this->window->Hide ();
	this->window->Delete ();
	this->actors->Delete ();
	this->fields->Delete ();
#if 0
	this->bactor->Delete ();
	this->bfield->Delete ();
#endif
	this->button_connect->Delete ();
	this->button_cancel->Delete ();
	
	this->window = 0;
	this->actors = 0;
	this->fields = 0;
	this->bactor = 0;
	this->bfield = 0;
	this->button_connect = 0;
	this->button_cancel = 0;
	
	this->Delete ();
}
