/*
 *	springbox.h
 *
 *	Spring box classes used to represent and edit the spring constants
 *	attached to a view.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_IB_SPRINGBOX_H_
#define	_IB_SPRINGBOX_H_

#ifndef	_OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

APP_EXPORT_CLASS (OPSpringBox);
APP_DEFINE_REF (OPSpringBox, OPView);
APP_DEFINE_ARRAY (OPSpringBox, OPView);

/****************************************************************/

class OPSpringBox : public OPView
{
protected:
	OPSpringBandRef			band;					//	associated spring band
	OPViewRef				view;					//	view being edited in band
	
	Bool					is_uptodate;			//	TRUE => following info is valid
	Bool					is_vertical;			//	TRUE => inspect vertical springs
	Bool					is_valid;				//	TRUE => valid springs
	OPSpringRef				s_before;				//	spring before (left or above)
	OPSpringRef				s_middle;				//	spring inside the view
	OPSpringRef				s_after;				//	spring after (right or below)
	
protected:
	OPSpringBox ();
	virtual ~ OPSpringBox ();
	
	virtual void UpdateSprings ();
	
public:
	virtual void Delete ();							/* OVERRIDDEN */
	
	virtual Bool SpringChanged ();
	virtual Bool SetSpringBand (OPSpringBandIn band);
	virtual Bool SetSpringView (OPViewIn view);
	
	virtual const OPSpringBandRef& ReturnSpringBand () const;
	virtual const OPViewRef& ReturnSpringView () const;
	
	virtual void Draw (OPGrailPortIn port, OPRectIn rect, OPDrawOp what = OP_DRAW_ALL); /* OVERRIDDEN */
	virtual void ButtonPress (OPEventIn event);		/* OVERRIDDEN */
	
	APP_DEFINE_CLASS (OPSpringBox, OPView);
};

/****************************************************************/

#endif /* _IB_SPRINGBOX_H_ */
