/*
 *	springedit.h
 *
 *	The Spring Edit class is used to edit a view's springs.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_IB_SPRINGEDIT_H_
#define	_IB_SPRINGEDIT_H_

#include "springbox.h"
#include "springview.h"

/****************************************************************/

APP_EXPORT_CLASS (OPSpringEdit);

APP_DEFINE_REF (OPSpringEdit, OPObject);
APP_DEFINE_ARRAY (OPSpringEdit, OPObject);

/****************************************************************/

class OPSpringEdit : public OPObject
{
protected:
	OPViewRef				edit;					//	object to edit
	OPViewRef				view;					//	main view of edition dialog
	
	OPButtonArray			buttons;				//	array of buttons
	OPSwitchArray			switches;				//	array of switches
	
	OPSpringBoxRef			spring_box;				//	spring box used for edition
	OPSpringViewRef			spring_view;			//	highlighted spring band
	OPSpringBoardRef		board;
	OPSpringBandRef			band;
	Count					band_num;
	Bool					is_vertical;
	
protected:
	OPSpringEdit ();
	virtual ~ OPSpringEdit ();
	
public:
	virtual void Delete ();							/* OVERRIDDEN */
	
	virtual Bool Initialise (OPObjectIn edit, OPCoord dx, OPCoord dy);
	virtual Bool Update (Bool find_view = FALSE);
	virtual Bool UpdateButtonEnable ();
	
	virtual const OPViewRef& ReturnView () const;
	
	virtual Bool DoBandNew ();
	virtual Bool DoBandSelect (int increment = 1);
	virtual Bool DoBandInsert ();
	virtual Bool DoBandRemove ();
	
	virtual Bool DoShowHorizontal ();
	virtual Bool DoShowVertical ();
	
	virtual Bool ActiveViewChanged (OPObjectIn tab, Card32 id);
	
	APP_DEFINE_CLASS (OPSpringEdit, OPObject);
};

/****************************************************************/

#endif /* _IB_SPRINGEDIT_H_ */
